# Test Data
TD_NAME = "name"
TD_IDS = "ids"

# Images
IMG_INTERVENTION_ID = "intervention_id"
IMG_INTERVENTION_ID = "intervention_id"
IMG_ANNOTATIONS = "annotations"
IMG_FRAME_NUMBER = "frame_number"
IMG_IS_FRAME = "is_frame"

# Shared
ORIGIN = "origin"
METADATA = "metadata"
